package demoapp;

import java.time.Instant;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class MemoryUsageMonitor implements AutoCloseable {

  private final ScheduledExecutorService pool;
  private final Deque<Long> sampleList;
  private final int maxSamples;

  public MemoryUsageMonitor(int maxSamples, long sampleDelayMillis) {
    this.sampleList = new ArrayDeque<>();
    this.maxSamples = maxSamples;
    this.pool = Executors.newScheduledThreadPool(1);

    pool.scheduleAtFixedRate(this::sample, 0, sampleDelayMillis, TimeUnit.MILLISECONDS);
  }

  public synchronized long getAverage() {
    if (sampleList.isEmpty())
      return -1;

    long sum = 0;
    for (long sample : sampleList) {
      sum += sample;
    }
    return sum / sampleList.size();
  }

  public void close() {
    pool.shutdown();
  }

  private synchronized void sample() {
    sampleList.addLast(Runtime.getRuntime().freeMemory());
    while (sampleList.size() > maxSamples)
      sampleList.removeFirst();

    System.out.println("Sampled at " + Instant.now());
  }

}
